/*-------------------------------------------------------------------------
 *
 * pg_subscription_d.h
 *    Macro definitions for pg_subscription
 *
 * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_SUBSCRIPTION_D_H
#define PG_SUBSCRIPTION_D_H

#define SubscriptionRelationId 6100
#define SubscriptionRelation_Rowtype_Id 6101

#define Anum_pg_subscription_oid 1
#define Anum_pg_subscription_subdbid 2
#define Anum_pg_subscription_subname 3
#define Anum_pg_subscription_subowner 4
#define Anum_pg_subscription_subenabled 5
#define Anum_pg_subscription_subbinary 6
#define Anum_pg_subscription_substream 7
#define Anum_pg_subscription_subconninfo 8
#define Anum_pg_subscription_subslotname 9
#define Anum_pg_subscription_subsynccommit 10
#define Anum_pg_subscription_subpublications 11

#define Natts_pg_subscription 11


#endif							/* PG_SUBSCRIPTION_D_H */
