/*-------------------------------------------------------------------------
 *
 * pg_init_privs_d.h
 *    Macro definitions for pg_init_privs
 *
 * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_INIT_PRIVS_D_H
#define PG_INIT_PRIVS_D_H

#define InitPrivsRelationId 3394

#define Anum_pg_init_privs_objoid 1
#define Anum_pg_init_privs_classoid 2
#define Anum_pg_init_privs_objsubid 3
#define Anum_pg_init_privs_privtype 4
#define Anum_pg_init_privs_initprivs 5

#define Natts_pg_init_privs 5


#endif							/* PG_INIT_PRIVS_D_H */
