/*-------------------------------------------------------------------------
 *
 * pg_statistic_ext_data_d.h
 *    Macro definitions for pg_statistic_ext_data
 *
 * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_STATISTIC_EXT_DATA_D_H
#define PG_STATISTIC_EXT_DATA_D_H

#define StatisticExtDataRelationId 3429

#define Anum_pg_statistic_ext_data_stxoid 1
#define Anum_pg_statistic_ext_data_stxdndistinct 2
#define Anum_pg_statistic_ext_data_stxddependencies 3
#define Anum_pg_statistic_ext_data_stxdmcv 4
#define Anum_pg_statistic_ext_data_stxdexpr 5

#define Natts_pg_statistic_ext_data 5


#endif							/* PG_STATISTIC_EXT_DATA_D_H */
