/*-------------------------------------------------------------------------
 *
 * pg_ts_dict_d.h
 *    Macro definitions for pg_ts_dict
 *
 * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_TS_DICT_D_H
#define PG_TS_DICT_D_H

#define TSDictionaryRelationId 3600

#define Anum_pg_ts_dict_oid 1
#define Anum_pg_ts_dict_dictname 2
#define Anum_pg_ts_dict_dictnamespace 3
#define Anum_pg_ts_dict_dictowner 4
#define Anum_pg_ts_dict_dicttemplate 5
#define Anum_pg_ts_dict_dictinitoption 6

#define Natts_pg_ts_dict 6


#endif							/* PG_TS_DICT_D_H */
